package jitas.core;

import java.io.Serializable;
import java.util.Map;

import jitas.exception.*;

/**
 * Create a Problem object which contains the necessary information.
 *  
 * @author	DeathMarch (c314g2)
 * @version	15 May 2007
 */
public class Problem  implements Comparable<Problem>, Serializable {
	
	private static final long serialVersionUID = 6656488745393963687L;
	private String problemStatement;
	private int id,difficulty;
	private Solution solution;
	private String shortName;
	private Map<String, String> additionalData;
	
	/**
	 * Constructor-- Initialize the variables
	 */
	public Problem (){
		id = 0;
		difficulty = 0;
		problemStatement = "";
		solution = new Solution();
		shortName = "";
	}
	
	/**
	 * Set the id of a Problem
	 * @param id The id of a problem
	 */
	public void setId(int id){
		this.id = id;
	}

	/**
	 * Set the difficulty of a Problem
	 * @param difficulty The difficulty level of a problem
	 */
	public void setDifficulty(int difficulty){
		this.difficulty = difficulty;
	}
	
	/**
	 * Set the problemStatement of a Problem
	 * @param problemStatement The statement of a problem
	 */
	public void setProblemStatement(String problemStatement){
		this.problemStatement = problemStatement;
	}
	
	/**
	 * Set the solution of a Problem
	 * @param solution The correct solution of problem
	 */
	public void setSolution(Solution solution){
		this.solution = solution;
	}
	
	/**
	 * Set the solution to a problem
	 * @param s The solution of a problem in string format
	 * @throws InvalidSolutionException
	 */
	public void setSolution(String s) {
		solution = new Solution(s);
	}
	
	/**
	 * Get the id of a Problem
	 * @return id The id of a problem
	 */
	public int getId(){
		return id;
	}

	/**
	 * Get the difficulty of a Problem
	 * @return difficulty The difficulty of a problem
	 */
	public int getDifficulty(){
		return difficulty;
	}
	
	/**
	 * Get the problemStatement of a Problem
	 * @return A copy of the statement of a problem
	 */
	public String getProblemStatement(){
		return new String(problemStatement);
	}
	
	/**
	 * Get the solution of a Problem
	 * @return solution The solution of a problem
	 */
	public Solution getSolution(){
		return solution;
	}
	
	/**
	 * Convert a problem to the XML format
	 * @return the final string in XML format
	 */
	public String toString(){
		StringBuffer result = new StringBuffer();
		result.append("<problem>");
		result.append("<id>"+this.id+"</id>");
		result.append("<difficulty>"+this.difficulty+"</difficulty>");
		result.append("<shortName>"+this.shortName+"</shortName>");
		result.append("<problemStatement>"+this.problemStatement+"</problemStatement>");
		result.append("<solution>"+this.solution+"</solution>");
		result.append("</problem>");
		return result.toString();
	}
	
	/**
	 * Standard equals override, based on the Problem id
	 * @return True if the objects are equal
	 */
	public boolean equals(Object object) {
		if (object instanceof Problem) {
			Problem other = (Problem) object;
			return this.getId() == other.getId();
		} else {
			return false;
		}
	}
	
	/**
	 * Standard compareTo override, based on the Problem id
	 * @param object the Object to be compared
	 * @return a negative integer, zero, or a positive integer as this object
	 * is less than, equal to, or greater than the specified object.
	 * @throws ClassCastException
	 */
	public int compareTo(Problem object) throws ClassCastException {
		if (object instanceof Problem) {
			Problem other = (Problem) object;
			return this.id - other.getId();
		} else {
			throw new ClassCastException();
		}
	}
	
	/**
	 * Standard hashCode override, based on the Problem id
	 * @return The hash code of a problem
	 */
	public int hashCode() {
		return id * 32 -4;
	}

	/**
	 * Get the short name of a problem
	 * @return shortName The short name of a problem
	 */
	public String getShortName() {
		return shortName;
	}
	
	/**
	 * Set the short name of a problem
	 * @param shortName The short name of a problem
	 */
	public void setShortName(String shortName) {
		this.shortName = shortName;
	}
	
	/**
	 * Get the additional data associated with a problem
	 * @return additionalData The additional data associated with a problem
	 */
	public Map<String, String> getAdditionalData() {
		return additionalData;
	}

	/**
	 * Set the additional data associated with a problem
	 * @param additionalData The additional data associated with a problem
	 */
	public void setAdditionalData(Map<String, String> additionalData) {
		this.additionalData = additionalData;
	}
}
