package jitas.util;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import sun.misc.BASE64Encoder;

/**
 * The encryptor class encrypts the real password and return it back for
 * saving and authentication purpose. 
 * @author DeathMarch
 * @version 05 July 2007
 */
public class Encryptor {
	/*Follow the singleton design pattern that takes only one instance*/
	private static Encryptor encryptor;
	
	/**
	 * Encrypt the password using SHA algorithm
	 * @param password the password that needs to be encrypted
	 * @return hash Encrypted password
	 */
	public String encrypt(String password){
		MessageDigest md = null;
		try{
			 md = MessageDigest.getInstance("SHA-512");
		} catch(NoSuchAlgorithmException e) {
			return null;
		}
		try{
			md.update(password.getBytes("UTF-8"));
		}catch(UnsupportedEncodingException e){
			return null;
		}
		byte raw[] = md.digest();
		String hash = (new BASE64Encoder()).encode(raw);
		return hash;
	}
	
	/**
	 * Create a new object if it does not exist or return the existing instance
	 * @return An instance of an encryptor
	 */
	public static synchronized Encryptor getEncryptor(){
	    if(encryptor == null){
	      return new Encryptor();
	    } else {
	      return encryptor;
	    }
	  }
}
